import 'dart:async';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:uuid/uuid.dart';
import '../utils/time_formatter.dart';
import '../utils/calculations.dart';
import 'config_screen.dart';
import 'about_screen.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import '../components/navigation_bar.dart';

class HomeScreen extends StatefulWidget {
  const HomeScreen({super.key});

  @override
  _HomeScreenState createState() => _HomeScreenState();
}

class _HomeScreenState extends State<HomeScreen> {
  DateTime? lastSmoked;
  bool isFirstTime = true;
  bool _friendsEnabled = true;

  int packetsPerWeek = 0;
  int cigarettesPerPacket = 0;

  double moneySaved = 0.0;
  int cigarettesAvoided = 0;

  late Timer timer;

  @override
  void initState() {
    super.initState();
    _generateAndSaveUserId();
    _initializeData();
    // Periodically update the UI every second
    timer = Timer.periodic(Duration(seconds: 1), (_) => _updateCalculations());
  }

  @override
  void dispose() {
    timer.cancel();
    super.dispose();
  }

  Future<void> _generateAndSaveUserId() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String? userId = prefs.getString('userId');
    if (userId == null) {
      var uuid = Uuid();
      userId = uuid.v4();
      await prefs.setString('userId', userId);
    }
  }

  Future<void> _initializeData() async {
    await _loadConfigData();
    await _loadLastSmoked();
    _updateCalculations();
  }

  Future<void> _loadLastSmoked() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String? storedTime = prefs.getString('lastSmoked');
    bool hasStarted = prefs.getBool('hasStarted') ?? false;

    if (storedTime != null && hasStarted) {
      setState(() {
        lastSmoked = DateTime.parse(storedTime);
        isFirstTime = false;
      });
    }
  }

  Future<void> _loadConfigData() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    setState(() {
      packetsPerWeek = prefs.getInt('packetsPerWeek') ?? 0;
      cigarettesPerPacket = prefs.getInt('cigarettesPerPacket') ?? 0;
      _friendsEnabled = prefs.getBool('friendsEnabled') ?? true;
    });
  }

  Future<void> _updateCalculations() async {
    if (lastSmoked != null) {
      double saved = await Calculations.calculateMoneySaved(lastSmoked!);
      int avoided = await Calculations.calculateCigarettesAvoided(
        lastSmoked!,
        packetsPerWeek,
        cigarettesPerPacket,
      );

      setState(() {
        moneySaved = saved;
        cigarettesAvoided = avoided;
      });
    }
  }

  void _startCounter() {
    setState(() {
      lastSmoked = DateTime.now();
      isFirstTime = false;
    });
    _saveLastSmoked();
    _updateCalculations();
  }

  Future<void> _saveLastSmoked() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await prefs.setString('lastSmoked', lastSmoked!.toIso8601String());
    await prefs.setBool('hasStarted', true);
  }

  @override
  Widget build(BuildContext context) {
    final duration = lastSmoked != null
        ? DateTime.now().difference(lastSmoked!)
        : Duration();

    final timeComponents = {
      'days': duration.inDays,
      'hours': duration.inHours % 24,
      'minutes': duration.inMinutes % 60
    };

    return Scaffold(
      appBar: AppBar(
        title: Text('ExHale'),
        actions: [
          PopupMenuButton<String>(
            onSelected: (value) {
              if (value == 'config') {
                Navigator.push(
                  context,
                  MaterialPageRoute(builder: (context) => ConfigScreen()),
                ).then((_) => _initializeData());
              } else if (value == 'about') {
                Navigator.push(
                  context,
                  MaterialPageRoute(builder: (context) => AboutScreen()),
                );
              }
            },
            itemBuilder: (BuildContext context) => [
              PopupMenuItem(
                value: 'config',
                child: Text(AppLocalizations.of(context)!.homeMenuConfig),
              ),
              PopupMenuItem(
                value: 'about',
                child: Text(AppLocalizations.of(context)!.homeMenuAbout),
              ),
            ],
          ),
        ],
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Text(
              'ExHale',
              style: TextStyle(fontSize: 40, fontWeight: FontWeight.bold),
            ),
            SizedBox(height: 20),
            Text(
              formatTime(timeComponents),
              style: TextStyle(fontSize: 30, fontWeight: FontWeight.bold),
            ),
            SizedBox(height: 20),
            Text(
              AppLocalizations.of(context)!.homeCigarettesAvoidedText(
                cigarettesAvoided.toString(),
              ),
              style: TextStyle(fontSize: 20),
            ),
            SizedBox(height: 10),
            Text(
              AppLocalizations.of(context)!.homeMoneySavedText(
                moneySaved.toStringAsFixed(2),
              ),
              style: TextStyle(fontSize: 20),
            ),
            SizedBox(height: 20),
            if (isFirstTime)
              ElevatedButton(
                onPressed: _startCounter,
                child: Text(AppLocalizations.of(context)!.homeStartCounterText),
              )
            else
              ElevatedButton(
                onPressed: _startCounter,
                child: Text(AppLocalizations.of(context)!.homeRestartProgressText),
              ),
          ],
        ),
      ),
      bottomNavigationBar: const CustomNavigationBar(currentIndex: 0),
    );
  }
}
